package dev.fyloz.colorrecipesexplorer.config.security.filters

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.properties.CreSecurityProperties
import dev.fyloz.colorrecipesexplorer.config.security.GroupAuthenticationToken
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDetails
import dev.fyloz.colorrecipesexplorer.logic.account.GroupTokenLogic
import dev.fyloz.colorrecipesexplorer.logic.account.JwtLogic
import org.springframework.security.authentication.AuthenticationManager
import org.springframework.security.authentication.BadCredentialsException
import org.springframework.security.core.Authentication
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

class GroupTokenAuthenticationFilter(
    jwtLogic: JwtLogic,
    securityProperties: CreSecurityProperties,
    private val groupTokenLogic: GroupTokenLogic,
    private val authManager: AuthenticationManager
) : JwtAuthenticationFilter(Constants.ControllerPaths.GROUP_LOGIN, securityProperties, jwtLogic) {
    override fun attemptAuthentication(request: HttpServletRequest, response: HttpServletResponse): Authentication {
        val groupTokenId = groupTokenLogic.getIdForRequest(request)
            ?: throw BadCredentialsException("Required group token cookie was not present")

        logger.debug("Login attempt for group token $groupTokenId")
        return authManager.authenticate(GroupAuthenticationToken(groupTokenId))
    }

    override fun afterSuccessfulAuthentication(userDetails: UserDetails) {
        logger.info("Successful login for group id '${userDetails.group!!.id}' using token '${userDetails.id}' (${userDetails.username})")
    }
}
