package dev.fyloz.colorrecipesexplorer.model.account

import org.springframework.security.core.GrantedAuthority
import org.springframework.security.core.authority.SimpleGrantedAuthority

enum class Permission(
    val id: Int,
    private val impliedPermissions: List<Permission> = listOf(),
    val deprecated: Boolean = false
) {
    READ_FILE(0),
    WRITE_FILE(1, listOf(READ_FILE)),

    VIEW_RECIPES(2, listOf(READ_FILE)),
    VIEW_CATALOG(3, listOf(READ_FILE)),
    VIEW_USERS(4),

    EDIT_RECIPES_PUBLIC_DATA(5, listOf(VIEW_RECIPES)),
    EDIT_RECIPES(6, listOf(EDIT_RECIPES_PUBLIC_DATA, WRITE_FILE)),
    EDIT_MATERIALS(7, listOf(VIEW_CATALOG, WRITE_FILE)),
    EDIT_MATERIAL_TYPES(8, listOf(VIEW_CATALOG)),
    EDIT_COMPANIES(9, listOf(VIEW_CATALOG)),
    EDIT_USERS(10, listOf(VIEW_USERS)),
    EDIT_CATALOG(11, listOf(EDIT_MATERIALS, EDIT_MATERIAL_TYPES, EDIT_COMPANIES)),

    VIEW_TOUCH_UP_KITS(12),
    EDIT_TOUCH_UP_KITS(13, listOf(VIEW_TOUCH_UP_KITS)),

    PRINT_MIXES(14, listOf(VIEW_RECIPES)),
    ADD_TO_INVENTORY(15, listOf(VIEW_CATALOG)),
    DEDUCT_FROM_INVENTORY(16, listOf(VIEW_RECIPES)),

    ADMIN(
        17,
        listOf(
            EDIT_RECIPES,
            EDIT_CATALOG,
            EDIT_USERS,

            EDIT_TOUCH_UP_KITS,

            PRINT_MIXES,
            ADD_TO_INVENTORY,
            DEDUCT_FROM_INVENTORY,
        )
    ),

    // deprecated permissions
    VIEW_RECIPE(101, listOf(VIEW_RECIPES), true),
    VIEW_MATERIAL(102, listOf(VIEW_CATALOG), true),
    VIEW_MATERIAL_TYPE(103, listOf(VIEW_CATALOG), true),
    VIEW_COMPANY(104, listOf(VIEW_CATALOG), true),
    VIEW(105, listOf(VIEW_RECIPES, VIEW_CATALOG), true),
    VIEW_EMPLOYEE(106, listOf(VIEW_USERS), true),
    VIEW_EMPLOYEE_GROUP(107, listOf(VIEW_USERS), true),

    EDIT_RECIPE(108, listOf(EDIT_RECIPES), true),
    EDIT_MATERIAL(109, listOf(EDIT_MATERIALS), true),
    EDIT_MATERIAL_TYPE(110, listOf(EDIT_MATERIAL_TYPES), true),
    EDIT_COMPANY(111, listOf(EDIT_COMPANIES), true),
    EDIT(112, listOf(EDIT_RECIPES, EDIT_CATALOG), true),
    EDIT_EMPLOYEE(113, listOf(EDIT_USERS), true),
    EDIT_EMPLOYEE_PASSWORD(114, listOf(EDIT_USERS), true),
    EDIT_EMPLOYEE_GROUP(115, listOf(EDIT_USERS), true),

    REMOVE_FILE(116, listOf(WRITE_FILE), true),
    GENERATE_TOUCH_UP_KIT(117, listOf(VIEW_TOUCH_UP_KITS), true),

    REMOVE_RECIPES(118, listOf(EDIT_RECIPES, REMOVE_FILE), true),
    REMOVE_MATERIALS(119, listOf(EDIT_MATERIALS, REMOVE_FILE), true),
    REMOVE_MATERIAL_TYPES(120, listOf(EDIT_MATERIAL_TYPES), true),
    REMOVE_COMPANIES(121, listOf(EDIT_COMPANIES), true),
    REMOVE_USERS(122, listOf(EDIT_USERS), true),
    REMOVE_CATALOG(123, listOf(REMOVE_MATERIALS, REMOVE_MATERIAL_TYPES, REMOVE_COMPANIES), true),

    REMOVE_RECIPE(124, listOf(REMOVE_RECIPES), true),
    REMOVE_MATERIAL(125, listOf(REMOVE_MATERIALS), true),
    REMOVE_MATERIAL_TYPE(126, listOf(REMOVE_MATERIAL_TYPES), true),
    REMOVE_COMPANY(127, listOf(REMOVE_COMPANIES), true),
    REMOVE(128, listOf(REMOVE_RECIPES, REMOVE_CATALOG), true),
    REMOVE_EMPLOYEE(129, listOf(REMOVE_USERS), true),
    REMOVE_EMPLOYEE_GROUP(130, listOf(REMOVE_USERS), true),

    SET_BROWSER_DEFAULT_GROUP(131, listOf(VIEW_USERS), true),
    ;

    operator fun contains(permission: Permission): Boolean {
        return permission == this || impliedPermissions.any { permission in it }
    }

    fun flat(): Iterable<Permission> {
        return mutableSetOf(this).apply {
            impliedPermissions.forEach {
                addAll(it.flat())
            }
        }
    }

    /** Converts the given permission to a [GrantedAuthority]. */
    fun toAuthority(): GrantedAuthority {
        return SimpleGrantedAuthority(name)
    }
}
