package dev.fyloz.colorrecipesexplorer.service.account

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.account.GroupTokenDto
import dev.fyloz.colorrecipesexplorer.model.account.GroupToken
import dev.fyloz.colorrecipesexplorer.repository.GroupTokenRepository
import java.util.UUID

interface GroupTokenService {
    fun existsById(id: UUID): Boolean
    fun existsByName(name: String): Boolean
    fun getAll(): Collection<GroupTokenDto>
    fun getById(id: UUID): GroupTokenDto?
    fun save(token: GroupTokenDto): GroupTokenDto
    fun deleteById(id: UUID)

    fun toDto(entity: GroupToken): GroupTokenDto
    fun toEntity(dto: GroupTokenDto): GroupToken
}

@ServiceComponent
class DefaultGroupTokenService(private val repository: GroupTokenRepository, private val groupService: GroupService) :
    GroupTokenService {
    override fun existsById(id: UUID) = repository.existsById(id)
    override fun existsByName(name: String) = repository.existsByNameAndIsDeletedIsFalse(name)

    override fun getAll() = repository.findAllByIsDeletedIsFalse().map(::toDto)

    override fun getById(id: UUID): GroupTokenDto? {
        val entity = repository.findByIdAndIsDeletedIsFalse(id)

        return if (entity != null) toDto(entity) else null
    }

    override fun save(token: GroupTokenDto): GroupTokenDto {
        val entity = repository.save(toEntity(token))
        return toDto(entity)
    }

    override fun deleteById(id: UUID) = repository.deleteById(id)

    override fun toDto(entity: GroupToken) =
        GroupTokenDto(entity.id, entity.name, entity.isValid, entity.isDeleted, groupService.toDto(entity.group))

    override fun toEntity(dto: GroupTokenDto) =
        GroupToken(dto.id, dto.name, dto.enabled, dto.isDeleted, groupService.toEntity(dto.group))
}
